@@ -30,24 +30,17 @@ Anything coin-specific should go in this file and be subclassed where
 necessary for appropriate handling.
 '''
 
-from collections import namedtuple
 import re
 from decimal import Decimal
 from hashlib import sha256
 
-import electrumx.lib.util as util
-from electrumx.lib.hash import Base58, double_sha256, hash_to_hex_str
+from electrumx.lib import util
+from electrumx.lib.hash import Base58, double_sha256
 from electrumx.lib.hash import HASHX_LEN
 from electrumx.lib.script import ScriptPubKey
-import electrumx.lib.tx as lib_tx
-import electrumx.server.block_processor as block_proc
-import electrumx.server.daemon as daemon
 from electrumx.server.session import ElectrumX
 
 
-Block = namedtuple("Block", "raw header transactions")
-
-
 class CoinError(Exception):
     '''Exception raised for coin-related errors.'''
 
@@ -63,9 +56,6 @@ class Coin:
     VALUE_PER_COIN = 100000000
     SESSIONCLS = ElectrumX
     DEFAULT_MAX_SEND = 1000000
-    DESERIALIZER = lib_tx.Deserializer
-    DAEMON = daemon.Daemon
-    BLOCK_PROCESSOR = block_proc.BlockProcessor
     P2PKH_VERBYTE = bytes.fromhex("00")
     P2SH_VERBYTES = [bytes.fromhex("05")]
     RPC_PORT = 8332
@@ -81,7 +71,7 @@ class Coin:
         '''Return a coin class given name and network.
 
         Raise an exception if unrecognised.'''
-        req_attrs = ['TX_COUNT', 'TX_COUNT_HEIGHT', 'TX_PER_BLOCK']
+        req_attrs = ['CHAIN_SIZE', 'CHAIN_SIZE_HEIGHT', 'AVG_BLOCK_SIZE']
         for coin in util.subclasses(Coin):
             if (coin.NAME.lower() == name.lower() and
                     coin.NET.lower() == net.lower()):
@@ -108,26 +98,6 @@ class Coin:
             url = 'http://' + url
         return url + '/'
 
-    @classmethod
-    def max_fetch_blocks(cls, height):
-        if height < 130000:
-            return 1000
-        return 100
-
-    @classmethod
-    def genesis_block(cls, block):
-        '''Check the Genesis block is the right one for this coin.
-
-        Return the block less its unspendable coinbase.
-        '''
-        header = block[:80]
-        header_hex_hash = hash_to_hex_str(cls.header_hash(header))
-        if header_hex_hash != cls.GENESIS_HASH:
-            raise CoinError('genesis block has hash {} expected {}'
-                            .format(header_hex_hash, cls.GENESIS_HASH))
-
-        return header + bytes(1)
-
     @classmethod
     def hashX_from_script(cls, script):
         '''Returns a hashX from a script.'''
@@ -177,13 +147,6 @@ class Coin:
         '''Given a header return previous hash'''
         return header[4:36]
 
-    @classmethod
-    def block(cls, raw_block):
-        '''Return a Block namedtuple given a raw block and its height.'''
-        header = raw_block[:80]
-        txs = cls.DESERIALIZER(raw_block, start=len(header)).read_tx_block()
-        return Block(raw_block, header, txs)
-
     @classmethod
     def decimal_value(cls, value):
         '''Return the number of standard coin units as a Decimal given a
@@ -193,17 +156,24 @@ class Coin:
         '''
         return Decimal(value) / cls.VALUE_PER_COIN
 
+    @classmethod
+    def prefetch_limit(cls, height):
+        if height <= 650_000:
+            return 100
+        return 10
+
 
 class BitcoinSV(Coin):
     NAME = "BitcoinSV"
-    TX_COUNT = 267318795
-    TX_COUNT_HEIGHT = 557037
-    TX_PER_BLOCK = 400
+    CHAIN_SIZE = 1_623_944_264_227
+    CHAIN_SIZE_HEIGHT = 709_728
+    AVG_BLOCK_SIZE = 150_000_000
     PEERS = [
         'electrumx.bitcoinsv.io s',
         'satoshi.vision.cash s',
         'sv.usebsv.com s t',
         'sv.satoshi.io s t',
+        'akiba.chainbow.io s t',
     ]
     GENESIS_ACTIVATION = 620_538
 
@@ -217,9 +187,10 @@ class BitcoinTestnetMixin:
     GENESIS_HASH = ('000000000933ea01ad0ee984209779ba'
                     'aec3ced90fa3f408719526f8d77f4943')
     REORG_LIMIT = 8000
-    TX_COUNT = 12242438
-    TX_COUNT_HEIGHT = 1035428
-    TX_PER_BLOCK = 21
+    CHAIN_SIZE = 26_584_216_544
+    CHAIN_SIZE_HEIGHT = 1_454_438
+    AVG_BLOCK_SIZE = 200_000
+
     RPC_PORT = 18332
     PEER_DEFAULT_PORTS = {'t': '51001', 's': '51002'}
 
@@ -238,16 +209,14 @@ class BitcoinSVScalingTestnet(BitcoinSVTestnet):
     PEERS = [
         'stn-server.electrumsv.io t51001 s51002',
     ]
-    TX_COUNT = 2015
-    TX_COUNT_HEIGHT = 5711
-    TX_PER_BLOCK = 5000
+    CHAIN_SIZE = 20_000
+    CHAIN_SIZE_HEIGHT = 100
+    AVG_BLOCK_SIZE = 2_000_000_000
     GENESIS_ACTIVATION = 14_896
 
     @classmethod
-    def max_fetch_blocks(cls, height):
-        if height <= 10:
-            return 100
-        return 3
+    def prefetch_limit(cls, height):
+        return 8
 
 
 class BitcoinSVRegtest(BitcoinSVTestnet):
@@ -255,8 +224,9 @@ class BitcoinSVRegtest(BitcoinSVTestnet):
     GENESIS_HASH = ('0f9188f13cb7b2c71f2a335e3a4fc328'
                     'bf5beb436012afca590b1a11466e2206')
     PEERS = []
-    TX_COUNT = 1
-    TX_COUNT_HEIGHT = 1
+    CHAIN_SIZE = 20_000
+    CHAIN_SIZE_HEIGHT = 100
+    AVG_BLOCK_SIZE = 1_000_000
     GENESIS_ACTIVATION = 10_000
 
 
