@@ -10,8 +10,10 @@ from asyncio import Event
 from aiorpcx import _version as aiorpcx_version, TaskGroup
 
 import electrumx
+import electrumx.server.block_processor as block_proc
 from electrumx.lib.server_base import ServerBase
 from electrumx.lib.util import version_string
+from electrumx.server.daemon import Daemon
 from electrumx.server.db import DB
 from electrumx.server.mempool import MemPool, MemPoolAPI
 from electrumx.server.session import SessionManager
@@ -98,16 +100,14 @@ class Controller(ServerBase):
         self.logger.info(f'reorg limit is {env.reorg_limit:,d} blocks')
 
         notifications = Notifications()
-        Daemon = env.coin.DAEMON
-        BlockProcessor = env.coin.BLOCK_PROCESSOR
 
         async with Daemon(env.coin, env.daemon_url) as daemon:
             db = DB(env)
-            bp = BlockProcessor(env, db, daemon, notifications)
+            bp = block_proc.BlockProcessor(env, db, daemon, notifications)
 
             # Set notifications up to implement the MemPoolAPI
             def get_db_height():
-                return db.db_height
+                return db.state.height
             notifications.height = daemon.height
             notifications.db_height = get_db_height
             notifications.cached_height = daemon.cached_height
@@ -135,6 +135,9 @@ class Controller(ServerBase):
             async with TaskGroup() as group:
                 await group.spawn(session_mgr.serve(notifications, mempool_event))
                 await group.spawn(bp.fetch_and_process_blocks(caught_up_event))
+                await group.spawn(bp.check_cache_size_loop())
                 await group.spawn(wait_for_catchup())
 
-            raise group.exception
+                async for task in group:
+                    if not task.cancelled():
+                        task.result()
