@@ -17,7 +17,7 @@ from collections import defaultdict, Counter
 
 from aiorpcx import (connect_rs, RPCSession, SOCKSProxy, Notification, handler_invocation,
                      SOCKSError, TaskTimeout, TaskGroup, Event,
-                     sleep, ignore_after, CancelledError, RPCError, ProtocolError)
+                     sleep, ignore_after, RPCError, ProtocolError)
 
 from electrumx.lib.peer import Peer
 from electrumx.lib.util import class_logger
@@ -256,10 +256,6 @@ class PeerManager:
                                   f'({e.code})')
             except (OSError, SOCKSError, ConnectionError, TaskTimeout) as e:
                 self.logger.info(f'{peer_text} {e}')
-            except CancelledError:
-                # A send_request was cancelled
-                self.logger.error(f'{peer_text} dropped verification connection')
-                raise
 
         if is_good:
             # Monotonic time would be better, but last_good and last_try are
@@ -346,8 +342,9 @@ class PeerManager:
             peers_task = await g.spawn(self._send_peers_subscribe
                                        (session, peer))
 
-        # Propagate failed task exception
-        g.results    # pylint:disable=W0104
+            async for task in g:
+                if not task.cancelled():
+                    task.result()
 
         # Process reported peers if remote peer is good
         peers = peers_task.result()
@@ -364,7 +361,7 @@ class PeerManager:
         result = await session.send_request(message)
         assert_good(message, result, dict)
 
-        our_height = self.db.db_height
+        our_height = self.db.state.height
         their_height = result.get('height')
         if not isinstance(their_height, int):
             raise BadPeerError(f'invalid height {their_height}')
@@ -434,6 +431,10 @@ class PeerManager:
             await group.spawn(self._detect_proxy())
             await group.spawn(self._import_peers())
 
+            async for task in group:
+                if not task.cancelled():
+                    task.result()
+
     def info(self):
         '''The number of peers.'''
         self._set_peer_statuses()
